;;; otter-insert.el
;;; This file is part of otter-mode
;;; a mode for editing and running Otter input files

;;; Author: Holger Schauer <Holger.Schauer@gmd.de>

;;; last modified: Thu Dec  5 21:17:51 1996

;;; This file defines some very handy shortcuts 
;;; for inserting Otter commands.
;;; This is the advanced version which checks if 
;;; there is a corresponding section already there.
;;; If yes a modification takes place.

;;; now define some input shortcuts

; first let's start with inserting formula_lists
; generic function
(defun otter-insert-all-lists (list-type list-name)
"This makes the necessary checks and inserts lists. If the corresponding
 list is already present in the input file just a new line in the list
 is inserted."
(let* ((old-point (point))
       (string (concat list-type  "(" list-name ").")))
  ;; check if formula list is already there
  (goto-char (point-min))
  (cond ((search-forward-regexp 
	  (concat "^" string) (point-max) t)
	 (if (search-forward-regexp "^end_of_list.")
	     (beginning-of-line))
	 (open-line 1)
	 (next-line 0))
	(t (goto-char old-point) ;; it is not there already
	   (otter-move-out-of-list old-point)
	   (forward-line 1)
	   (beginning-of-line) ;; move to beginning of the current line
	   (insert string)
	   (insert " \n\n")
	   (insert "end_of_list. \n")
	   (line-move -2)
	   (beginning-of-line)))
  ))

(defun otter-move-out-of-list (point)
"Checks if point is inside a list of clauses or formlas. If it is moves
 point outside of it. Returns point."
 ;; if we can find a end_of_list. which is in front of any 
 ;; other list(name) and any formula_list(name) then we are
 ;; inside a list.
(let* ((eol-point (search-backward-regexp "^.end_of_list" (point-min) t))
       (lp-point 
	    (or (search-forward-regexp
		 "^list(" (point-max) t)
		(search-forward-regexp
		 "^formula_list(" (point-max) t)))
       (other-eol-point (if (null eol-point)
			  (progn 
			    (goto-char point)
			    (search-forward-regexp "^end_of_list."
						   (point-max) t))
			  nil)))
       ;; okay: now if eol-point is non-nil and lp-point is less than 
       ;; point we are inside a list - so let's move out
       ;; if we haven't found an eol-point and no lp-point we're done
       ;; anyway
       ;; if no eol-point but lp-point is set we have again to check
       ;; if we can find an eol-point that is nearer to point than
       ;; lp-point
  (cond ((and (null lp-point) ;; no list can be found at all !
	      (null eol-point)
	      (null other-eol-point))
	 point) 
	((and (null lp-point) ;; one singleton list and we're in it
	      (null eol-point)) ;; because other-eol-point is non-nil
	 (otter-insert-line-at-lap other-eol-point))
	((null lp-point)  ;; an end of list before us but no list follows
	 (otter-insert-line-at-lap eol-point))
	((and (null eol-point) ;; no end-of-list in front, but list follows
	      (not (null lp-point)))
	 (cond ((null other-eol-point)
		;; no end-of-list can't be found but there is an list() ??
		(goto-char lp-point)
		(beginning-of-line)
		(point))
	       ((< other-eol-point lp-point) ;; aha, we're inside a list
		;; because we have found an end_of_list after point
		;; and in front of lp-point
		(otter-insert-line-at-lap other-eol-point)) 
	       ((< lp-point other-eol-point) ;; normality
		(goto-char lp-point)   ;; a list(name) with a following(!)
		(beginning-of-line)    ;; end_of_list.
		(forward-line -1)
		(open-line 1)
		(forward-line 0)
		(point)))))))

; a little utility
(defun otter-insert-line-at-lap (point)
 "Insert a new blank line at line after the specified point."
 (goto-char point)
 (beginning-of-line)
 (forward-line 1)
 (open-line 1)
 (forward-line 0)
 (point)
)

; insert a formula list
(defun otter-insert-formula-list()
"Inserts the keyword formula_list and also end_of_list."
(interactive)
(cond ((equal major-mode 'otter-mode)
	(let* ((ch-list (completing-read "List: " otter-cl-f-list nil t)))
	  (otter-insert-all-lists "formula_list" ch-list)))))
	       
			   
		      

; insert lists
(defun otter-insert-list()
"Inserts the keyword formula_list and also end_of_list."
(interactive)
(cond ((equal major-mode 'otter-mode)
	(let* ((ch-list (completing-read "List: " otter-cl-f-list nil t)))
	       (otter-insert-all-lists "list" ch-list)))))


; insert weight_list
(defun otter-insert-weight-list()
"Inserts the keyword formula_list and also end_of_list."
(interactive)
(cond ((equal major-mode 'otter-mode)
	(let* ((ch-list (completing-read "List: " otter-weight-list nil t)))
	  (otter-insert-all-lists "weight_list" ch-list)))))



; insert flags - flag is asked for in minibuffer
; generic function for all other options
(defun otter-insert-option (sc)
"Inserts the keyword depending on argument SC (set/clear) and reads the flag from the minibuffer." 
(let*  ;; save point, read flag search for flag
  ((old-point (point)) ; save point for later work
   (ch-option (completing-read "Flag: " otter-options-list nil t))
   (setstring (concat "set(" ch-option ").")) ;; string for set(flag)
   (clearstring (concat "clear(" ch-option ").")) ;; string for clear(flag)
   (flag-cleared (progn ;; seach for clear(flag)
		   (goto-char (point-min))
		   (search-forward-regexp
		      (concat "^" clearstring) (point-max) t))))
			      
       (cond ((and flag-cleared (eq sc 'set))
               ;; flag cleared but we want to set it
	         (goto-char flag-cleared)
	         (search-backward-regexp (concat "^" clearstring))
	         (replace-match setstring))

	      ((and flag-cleared (eq sc 'clear)) t)
	      ;; cleared and we want to clear it

	      ((and (not flag-cleared)
	      ;; not cleared -check if it is set 	    
		    (progn
		      (goto-char (point-min))
		      (search-forward-regexp
		           (concat "^" setstring) (point-max) t)))

	               (cond ((eq sc 'clear)          ; we want to clear   
			       (replace-match clearstring))
				   
			     ((eq sc 'set) t)         ; we want to set it
			     (t nil)))

	      ((not flag-cleared) 
		; neither cleared not set
       	        ; return to point and ensure we're at a new line
 	        (goto-char old-point)
		(otter-move-out-of-list old-point)
	        (cond ((eq sc 'set)
		       (insert setstring))
		      ((eq sc 'clear)
		       (insert clearstring)))) ; not zu
	 ) ; cond ((and flag-cleared ... zu
)) ; let zu, end of otter-insert-option
	  

; set flags
(defun otter-insert-set-option ()
"Inserts the a set otter-option into the otter-input-file. Completing
 via minibuffer is possible."
(interactive)
(otter-insert-option 'set))

; clear flags 
(defun otter-insert-clear-option ()
"Inserts the keyword clear and reads the flag from the minibuffer."
(interactive)
(otter-insert-option 'clear))

; insert parameter
(defun otter-insert-parameter ()
"Fetches parameter and value from minibuffer and inserts them."
(interactive)
(let* ((ch-parameter (completing-read "Parameter: " otter-parm-list nil t))
;       (ch-value (read-expression "Value: ")))
       (ch-value (read-from-minibuffer "Value: "))
       (old-point (point)))

  ;; let's first check if the parameter is already defined. 
  (goto-char (point-min))
  (cond ((search-forward-regexp 
       	   (concat "^assign(" ch-parameter) (point-max) t)
           ;; It it is, just adjust the value.
	   (search-forward-regexp  ",.*)." (point-max) t)
	   (replace-match (concat "," ch-value ").")))
  ;; else: /parameter is still undefined
	(t (goto-char old-point)
	   (end-of-line)
	   (open-line 1)
	   (next-line 1)	    
	   (insert "assign(")
	   (insert (format "%s" ch-parameter))
	   (insert ", ")
	   (insert (format "%s" ch-value))
	   (insert ").")))))


; insert include-file-command
(defun otter-insert-include ()
"Inserts an input-command."
(interactive)
(let* ((ch-fn (read-file-name "File: "))
  ;; check if it is already there .. the include command I mean :-)
      (string (concat "include(" ch-fn ")."))
      (old-point (point)))
   (goto-char (point-min))    
   (cond ((search-forward-regexp 
	    (concat "^" string)  (point-max) t)
	  t) ;; already there - just return
	 (t (goto-char old-point)
	    (open-line 1)
	    (next-line 1)
	    (insert string)))))

; insert operator 
(defun otter-insert-op ()
"Inserts an operator declaration."
(interactive)
(let* ((ch-pred (read-from-minibuffer "Precedence: "))
;       (ch-pred (read-expression "Precedence: "))
       (ch-type (completing-read "Type: " otter-functor-types nil t))
;       (ch-name (read-expression "Name: ")))
       (ch-name (read-from-minibuffer "Name: "))
       (old-point (point))
       (string (concat "op(" ch-pred ", " ch-type ", " ch-name ").")))
  ;; check if operator definition is already there
  (goto-char (point-min))
  (cond ((search-forward-regexp
	   (concat "^" string) (point-max) t) t)
        (t (goto-char old-point)
	   (end-of-line)
	   (open-line 1)
	   (next-line 1)
	   (insert string)))))

; insert make_evaluable - search and rewriting is still missing here
(defun otter-insert-me ()
"Insert command make_evaluable(sym,eval-sym)."
(interactive)
(let* ((ch-sym (read-from-minibuffer "Symbol: "))
;       (ch-sym (read-expression "Symbol: "))
;       (ch-es (read-expression "Evaluable Symbol: ")))
       (ch-es (read-from-minibuffer "Evaluable Symbol: ")))
  (insert "make_evaluable(")
  (insert (format "%s" ch-sym))
  (insert ", ")
  (insert (format "%s" ch-es))
  (insert ").\n")))

(defun otter-insert-lex ()
"Insert a lex(symbol_list) command."
  (interactive)
  (let ((old-point (point)))
    (goto-char (point-min))
    (cond ((search-forward-regexp "lex(.*)." (point-max) t)
	   (backward-char 2))
	  (t (goto-char old-point)
	     (end-of-line)
	     (open-line 1)
	     (next-line 1)
	     (insert "lex().")
	     (backward-char 2)))))

(defun otter-insert-skolem ()
"Insert a lex(symbol_list) command."
  (interactive)
  (let ((old-point (point)))
    (goto-char (point-min))
    (cond ((search-forward-regexp "skolem(.*)." (point-max) t)
	   (backward-char 2))
	  (t (goto-char old-point)
	     (end-of-line)
	     (open-line 1)
	     (next-line 1)
	     (insert "skolem().")
	     (backward-char 2)))))

(defun otter-insert-lrpo ()
"Insert a lex(symbol_list) command."
  (interactive)
  (let ((old-point (point)))
    (goto-char (point-min))
    (cond ((search-forward-regexp "lrpo_multiset_status(.*)." (point-max) t)
	   (backward-char 2))
	  (t (goto-char old-point)
	     (end-of-line)
	     (open-line 1)
	     (next-line 1)
	     (insert "lrpo_multiset_status().")
	     (backward-char 2)))))



(provide 'otter-insert)
